/*
 * This file is part of SubLink.
 * Copyright (C) 2009  Jeferson Hultmann <hultmann@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

"use strict";

var EXPORTED_SYMBOLS = ["PUBLIC_replaceCore"];

Components.utils.import("resource://sublink-migreme-4.1.0/util.js");


function PUBLIC_replaceCore(cmdData, newUrl, browser) {
  var rv;
  if (cmdData.selectionStart !== cmdData.selectionEnd) {
    //selection
    var txt = cmdData.node.value;
    rv = {
      count: 1,
      text: txt.substr(0, cmdData.selectionStart) + newUrl + txt.substr(cmdData.selectionEnd),
      caretStart: cmdData.selectionStart,
      caretEnd: cmdData.selectionStart + newUrl.length
    };
  } else {
    rv = replaceAll(cmdData, newUrl);
  }


  if (updateNode(cmdData, rv)) {
    // matched text/node/tab not found
    Components.utils.import("resource://sublink-migreme-4.1.0/err.js");
    var box = notifBox.getNotification(browser);
    notifBox.showNotFoundNotification(box, newUrl);
  }
}


function replaceAll(cmdData, newUrl) {
  var inp = cmdData.node;
  var txt = inp.value;
  var qty = 0;
  var idxStart = 0;

  do {
    var idxUrl = txt.indexOf(cmdData.typed, idxStart);
    if (idxUrl > -1) {
      var wordIdx = findWordPosition(txt, idxUrl);
      var urlFound = txt.substr(wordIdx[0], wordIdx[1]);
      if (urlFound === cmdData.typed) {
        txt = txt.substr(0, wordIdx[0]) + newUrl + txt.substr(wordIdx[0] + wordIdx[1]);
        idxStart = wordIdx[0] + newUrl.length;
        qty++;
      } else {
        idxStart = wordIdx[0] + wordIdx[1];
      }
    }
  } while (idxUrl > -1);


  return {
    count: qty,
    text: txt,
    caretStart: idxStart,
    caretEnd: idxStart
  };
}


function updateNode(cmdData, rv) {
  if (rv.count === 0) {
    return true;
  }

  if ("rows" in cmdData.node) {
    var pos = cmdData.node.scrollTop;
    cmdData.node.value = rv.text;
    cmdData.node.scrollTop = pos; // restore scrollbar position
  } else {
    cmdData.node.value = rv.text;
  }
  try {
    cmdData.node.setSelectionRange(rv.caretStart,
                                   rv.caretEnd);
  } catch (ex) {
    // node/page is no longer available
    return true;
  }

  return false;
}
